use serenity::{
    all::Message,
    client::Context,
    framework::standard::{macros::command, Args, CommandResult},
};

use crate::db::user_image::UserImage;

#[command]
#[description = "Enable/Disable the auto meme answer"]
pub async fn enable(ctx: &Context, msg: &Message, mut args: Args) -> CommandResult {
    let guild = match msg.guild_id {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let mut user_in_db: UserImage =
        match UserImage::find_by_server_id_user_id(&guild.get(), &msg.author.id.get()).await {
            Ok(Some(user_in_db)) => user_in_db.clone(),
            Ok(None) => {
                let user_in_db = UserImage::new(guild.get(), msg.author.id.get(), false).unwrap();
                match user_in_db.create().await {
                    Ok(_) => user_in_db,
                    Err(e) => {
                        if let Err(why) = msg
                            .channel_id
                            .say(&ctx.http, "Couldn't create user in db")
                            .await
                        {
                            println!("Error sending message: {:?}", why)
                        }
                        println!("Error saving user image: {:?}", e);
                        return Ok(());
                    }
                }
            }
            Err(e) => {
                if let Err(why) = msg
                    .channel_id
                    .say(&ctx.http, "Error finding user image")
                    .await
                {
                    println!("Error sending message: {:?}", why)
                }
                println!("Error finding user image: {:?}", e);
                return Ok(());
            }
        };
    if let Ok(new_status) = args.single::<bool>() {
        if user_in_db.enable == new_status {
            if let Err(why) = msg
                .channel_id
                .say(&ctx.http, format!("Auto answer the same : {}", new_status))
                .await
            {
                println!("Error sending message: {:?}", why)
            }
            return Ok(());
        }
        user_in_db.enable = new_status;
        match user_in_db.update().await {
            Ok(_) => {
                if let Err(why) = msg
                    .channel_id
                    .say(&ctx.http, format!("Auto answer : {}", new_status))
                    .await
                {
                    println!("Error sending message: {:?}", why)
                }
            }
            Err(e) => {
                if let Err(why) = msg
                    .channel_id
                    .say(&ctx.http, "Couldn't update user in db")
                    .await
                {
                    println!("Error sending message: {:?}", why)
                }
                println!("Error saving user image: {:?}", e);
                return Ok(());
            }
        }
        return Ok(());
    } else {
        if let Err(why) = msg
            .channel_id
            .say(
                &ctx.http,
                format!("Auto answer enabled : {}", user_in_db.enable),
            )
            .await
        {
            println!("Error sending message: {:?}", why)
        }
    }

    Ok(())
}
