mod bot;
mod config;
mod db;
mod img;

use std::{process, time::Duration};

use actix_cors::Cors;
use actix_web::{App, HttpServer};
use bot::init::start_bot;
use config::parse_local_config;
use tokio::{sync::oneshot, time::sleep};

#[tokio::main] // or #[actix_web::main]
async fn main() -> std::io::Result<()> {
    let config = parse_local_config();
    let port = config.port;
    match db::init::init_db(config.persistence.clone()).await {
        Ok(_) => {}
        Err(e) => {
            println!("Error initializing database: {}", e);
            return Ok(());
        }
    }
    let (tx, rx) = oneshot::channel();
    start_bot(config.clone(), rx);
    HttpServer::new(|| {
        let cors = Cors::default()
            .allow_any_header()
            .allow_any_method()
            .allow_any_origin();
        App::new().wrap(cors)
    })
    .bind(("0.0.0.0", port))?
    .run()
    .await?;
    println!("API Server stopped.");
    tx.send(()).unwrap();
    println!("Signal sent to bot.");
    sleep(Duration::from_secs(2)).await;
    process::exit(1); // This is a workaround to stop the bot, it should be replaced by a better solution
}
